// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MainForm.cs" company="The Watcher">
//   Copyright (c) The Watcher Partial Rights Reserved.
//  This software is licensed under the MIT license. See license.txt for details.
// </copyright>
// <summary>
//   Code Named: PG-Ripper
//   Function  : Extracts Images posted on VB forums and attempts to fetch them to disk.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Ripper
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Drawing;
    using System.IO;
    using System.Linq;
    using System.Net;
    using System.Reflection;
    using System.Resources;
    using System.Text.RegularExpressions;
    using System.Threading;
    using System.Windows.Forms;
    using System.Xml.Serialization;

#if (!PGRIPPERX)
    using Microsoft.WindowsAPICodePack.Taskbar;
#endif
    using Ripper.Objects;

    /// <summary>
    /// The Main Form
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// All Settings
        /// </summary>
        public SettingBase userSettings = new SettingBase();

        /// <summary>
        /// String List of Urls to Rip
        /// </summary>
        private readonly List<string> ExtractUrls = new List<string>();

        /// <summary>
        /// Array List of all Stored (Ripped) Post Ids
        /// </summary>
        private readonly ArrayList sRippedPosts = new ArrayList();

#if (!PGRIPPERX)
        /// <summary>
        /// TaskBar Manager Instance
        /// </summary>
        private TaskbarManager windowsTaskbar;
#endif

        /// <summary>
        /// The Indexed Topic Lists
        /// </summary>
        private List<ImageInfo> indexedTopicsList;

        /// <summary>
        /// Indicates if Ripper Currently Parsing Jobs
        /// </summary>
        private bool parseActive;

        /// <summary>
        /// Indicates that users Stopping and Deleting Current Job
        /// </summary>
        private bool stopingJob;

        /// <summary>
        /// Indicates that Disc is full and Ripping stopped
        /// </summary>
        private bool fullDisc;

        /// <summary>
        /// Indicates if Ripper is Currently Ripping
        /// </summary>
        private bool working = true;

        /// <summary>
        /// Indicates if Ripper is Currently Closing the Program
        /// </summary>
        private bool ripperClosing;

        /// <summary>
        /// Indicator if the ripper is currently hidden in the tray
        /// </summary>
        private bool isHiddenInTray;

        /// <summary>
        /// The Last Download Folder
        /// </summary>
        private string lastDownFolder;

        /// <summary>
        /// Initializes a new instance of the <see cref="MainForm"/> class.
        /// </summary>
        public MainForm()
        {
#if (!PGRIPPERX)
            // Tray Menue
            trayMenu = new ContextMenu();

            MenuItem hide = new MenuItem("Hide PG-Ripper", this.HideClick);
            MenuItem sDownload = new MenuItem("Start Download", this.SDownloadClick);
            MenuItem exit = new MenuItem("Exit Program", this.ExitClick);

            trayMenu.MenuItems.Add(0, hide);
            trayMenu.MenuItems.Add(1, sDownload);
            trayMenu.MenuItems.Add(2, exit);

            // Tray Icon
            trayIcon = new NotifyIcon
                {
                    Text = "Right click for context menu",
                    Visible = false,
                    Icon = new Icon(GetType(), "App.ico"),
                    ContextMenu = trayMenu
                };

            this.trayIcon.MouseDoubleClick += this.HideClick;
#endif
            InitializeComponent();

            jobsList = new List<JobInfo>();
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="MainForm"/> is delete.
        /// </summary>
        /// <value>
        ///   <c>true</c> if delete; otherwise, <c>false</c>.
        /// </value>
        public static bool Delete { get; set; }

        /// <summary>
        /// Gets or sets the Delete Message
        /// </summary>
        public static string DeleteMessage { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether if the Browse Dialog is Already Open
        /// </summary>
        public bool IsBrowserOpen { get; set; }

        /// <summary>
        /// Gets or sets the Resource Manger that is used for Icons and Labels etc.
        /// </summary>
        public ResourceManager _ResourceManager { get; set; }

#if (!PGRIPPERX)
        /// <summary>
        /// the download click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void SDownloadClick(object sender, EventArgs e)
        {
            if (Clipboard.GetText().IndexOf(this.userSettings.CurrentForumUrl) < 0 ||
                this.parseActive)
            {
                return;
            }

            this.comboBox1.SelectedIndex = 2;

            this.textBox1.Text = Clipboard.GetText();

            if (InvokeRequired)
            {
                Invoke((MethodInvoker)this.EnqueueJob);
            }
            else
            {
                this.EnqueueJob();
            }
        }

        /// <summary>
        /// Exits Ripper
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void ExitClick(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Hides Ripper in tray
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void HideClick(object sender, EventArgs e)
        {
            this.Hide();
            this.isHiddenInTray = true;

            trayMenu.MenuItems.RemoveAt(0);
            MenuItem show = new MenuItem("Show PG-Ripper", this.ShowClick);
            trayMenu.MenuItems.Add(0, show);

            this.trayIcon.MouseDoubleClick -= this.HideClick;
            this.trayIcon.MouseDoubleClick += this.ShowClick;

            if (!this.userSettings.ShowPopUps)
            {
                return;
            }

            trayIcon.BalloonTipIcon = ToolTipIcon.Warning;
            trayIcon.BalloonTipTitle = "Hidden in Tray";
            trayIcon.BalloonTipText = "PG-Ripper is hidden in the Tray";
            trayIcon.ShowBalloonTip(10);
        }

        /// <summary>
        /// Shows the Ripper.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void ShowClick(object sender, EventArgs e)
        {
            this.Show();
            this.isHiddenInTray = false;

            this.trayMenu.MenuItems.RemoveAt(0);

            var hide = new MenuItem("Hide PG-Ripper", this.HideClick);
            this.trayMenu.MenuItems.Add(0, hide);

            this.trayIcon.MouseDoubleClick -= this.ShowClick;
            this.trayIcon.MouseDoubleClick += this.HideClick;

            if (this.WindowState == FormWindowState.Minimized)
            {
                this.WindowState = FormWindowState.Normal;
            }
        }
#endif

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        /// <param name="args">The args.</param>
        [STAThread]
        private static void Main(string[] args)
        {
            Application.SetCompatibleTextRenderingDefault(false);
            Application.EnableVisualStyles();

            try
            {
                CacheController.Xform = new MainForm();
                ProcessArgs(args, CacheController.Xform);

                Application.Run(CacheController.Xform);
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Processes the args.
        /// </summary>
        /// <param name="args">The args.</param>
        /// <param name="aForm">A form.</param>
        private static void ProcessArgs(ICollection<string> args, Form aForm)
        {
            if (args.Count == 0)
            {
                return;
            }

            foreach (string curArg in args.Select(t => t.ToUpper()))
            {
                string aValue;
                if (CheckArg("MIN", curArg, out aValue))
                {
                    aForm.WindowState = FormWindowState.Minimized;
                }
            }
        }

        /// <summary>
        /// Checks the arg.
        /// </summary>
        /// <param name="aExpected">
        /// A expected.
        /// </param>
        /// <param name="aSupplied">
        /// A supplied.
        /// </param>
        /// <param name="aSuppliedValue">
        /// A supplied value.
        /// </param>
        /// <returns>
        /// The check arg.
        /// </returns>
        private static bool CheckArg(string aExpected, string aSupplied, out string aSuppliedValue)
        {
            int equalsPos = aSupplied.IndexOf("=");
            string lSupplied;
            if (equalsPos > -1)
            {
                aSuppliedValue = aSupplied.Substring(equalsPos + 1);
                lSupplied = aSupplied.Substring(0, equalsPos);
            }
            else
            {
                aSuppliedValue = string.Empty;
                lSupplied = aSupplied;
            }

            return lSupplied == aExpected || lSupplied == string.Format("/{0}", aExpected) || lSupplied == string.Format("\\{0}", aExpected);
        }

        /// <summary>
        /// Checks the Download Folder of the Current Finished Job, if Empty delete the folder.
        /// </summary>
        /// <param name="checkFolder">The check folder.</param>
        private static void CheckCurJobFolder(string checkFolder)
        {
            if (!Directory.Exists(checkFolder))
            {
                return;
            }

            if (Directory.GetFiles(checkFolder).Length == 0)
            {
                Directory.Delete(checkFolder);
            }
        }

        /// <summary>
        /// Loads the settings.
        /// </summary>
        private void LoadSettings()
        {
            this.userSettings = Utility.LoadSettings();

            // Max. Threads
            this.userSettings.ThreadLimit = this.userSettings.ThreadLimit == -1 ? 3 : this.userSettings.ThreadLimit;
            ThreadManager.GetInstance().SetThreadThreshHold(this.userSettings.ThreadLimit);

            if (string.IsNullOrEmpty(this.userSettings.DownloadFolder))
            {
                this.userSettings.DownloadFolder = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);

                this.DownloadFolder.Text = this.userSettings.DownloadFolder;

                Utility.SaveSettings(this.userSettings);
            }

            this.DownloadFolder.Text = this.userSettings.DownloadFolder;

            // Load "Download Options"
            try
            {
                this.comboBox1.SelectedIndex = Convert.ToInt32(this.userSettings.DownloadOptions);
            }
            catch (Exception)
            {
                this.userSettings.DownloadOptions = "0";
                this.comboBox1.SelectedIndex = 0;
            }

            this.TopMost = this.userSettings.TopMost;

            // Load Language Setting
            try
            {
                switch (this.userSettings.Language)
                {
                    case "de-DE":
                        this._ResourceManager = new ResourceManager(
                            "Ripper.Languages.german", Assembly.GetExecutingAssembly());
                        break;
                    case "fr-FR":
                        this._ResourceManager = new ResourceManager(
                            "Ripper.Languages.french", Assembly.GetExecutingAssembly());
                        break;
                    case "en-EN":
                        this._ResourceManager = new ResourceManager(
                            "Ripper.Languages.english", Assembly.GetExecutingAssembly());
                        break;
                    default:
                        this._ResourceManager = new ResourceManager(
                            "Ripper.Languages.english", Assembly.GetExecutingAssembly());
                        break;
                }

                this.AdjustCulture();
            }
            catch (Exception)
            {
                this._ResourceManager = new ResourceManager(
                    "Ripper.Languages.english", Assembly.GetExecutingAssembly());
            }

            switch (this.userSettings.AfterDownloads)
            {
                case 0:
                    this.doNothingToolStripMenuItem.Checked = true;
                    this.closeRipperToolStripMenuItem.Checked = false;
                    break;
                case 1:
                    this.doNothingToolStripMenuItem.Checked = false;
                    this.closeRipperToolStripMenuItem.Checked = true;
                    break;
            }

            // Load Show Last Download Image
            this.groupBox4.Visible = this.userSettings.ShowLastDownloaded;

            this.showLastImageToolStripMenuItem.Checked = this.userSettings.ShowLastDownloaded;
            this.useCliboardMonitoringToolStripMenuItem.Checked = this.userSettings.ClipBWatch;

            try
            {
                var accountNotFound =
                    this.userSettings.ForumsAccount.Where(
                        forumAccount => forumAccount.ForumURL.Equals(this.userSettings.CurrentForumUrl)).Any(
                            forumAccount =>
                            !string.IsNullOrEmpty(forumAccount.UserName)
                            && !string.IsNullOrEmpty(forumAccount.UserPassWord));

                if (accountNotFound)
                {
                    return;
                }

                var frmLgn = new Login();
                frmLgn.ShowDialog(this);

                if (this.bCameThroughCorrectLogin)
                {
                    return;
                }

                var result = TopMostMessageBox.Show(
                    this._ResourceManager.GetString("mbExit"),
                    this._ResourceManager.GetString("mbExitTtl"),
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question);

                if (result == DialogResult.Yes)
                {
                    Application.Exit();
                }
            }
            catch (Exception)
            {
                Login frmLgn = new Login();
                frmLgn.ShowDialog(this);

                if (this.bCameThroughCorrectLogin)
                {
                    this.CheckAccountMenu();

                    return;
                }

                var result = TopMostMessageBox.Show(
                    this._ResourceManager.GetString("mbExit"),
                    this._ResourceManager.GetString("mbExitTtl"),
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question);

                if (result == DialogResult.Yes)
                {
                    Application.Exit();
                }
            }
        }

        /// <summary>
        /// Adjusts the culture.
        /// </summary>
        private void AdjustCulture()
        {
            this.mStartDownloadBtn.Text = this._ResourceManager.GetString("btnStartDownload");
            this.groupBox1.Text = this._ResourceManager.GetString("downloadOptions");
            this.groupBox5.Text = string.Format("{0} (-):", this._ResourceManager.GetString("lblRippingQue"));
            this.stopCurrentThreads.Text = this._ResourceManager.GetString("btnStop");
            this.groupBox4.Text = this._ResourceManager.GetString("lblLastPicture");
            this.groupBox2.Text = this._ResourceManager.GetString("gbCurrently");
            this.mInvalidDestinationMsg = this._ResourceManager.GetString("mbInvalidDes");
            this.mIncorrectUrlMsg = this._ResourceManager.GetString("mbIncorrectURL");
            this.mNoThreadMsg = this._ResourceManager.GetString("mbNoThread");
            this.mNoPostMsg = this._ResourceManager.GetString("mbNoPost");
            this.mAlreadyQueuedMsg = this._ResourceManager.GetString("mbAlreadyQueued");
            this.mTNumericMsg = this._ResourceManager.GetString("mbTNumeric");

            // Menue
            this.fileToolStripMenuItem.Text = this._ResourceManager.GetString("MenuFile");
            this.exitToolStripMenuItem.Text = this._ResourceManager.GetString("MenuExit");

            this.accountsToolStripMenuItem.Text = this._ResourceManager.GetString("MenuAccounts");

            this.optionsToolStripMenuItem.Text = this._ResourceManager.GetString("MenuOptions");
            this.settingsToolStripMenuItem2.Text = this._ResourceManager.GetString("MenuSettings");
            this.showLastImageToolStripMenuItem.Text = this._ResourceManager.GetString("ShowLastDownloaded");
            this.useCliboardMonitoringToolStripMenuItem.Text = this._ResourceManager.GetString("clipboardWatch");
            this.afterDownloadsFinishedToolStripMenuItem.Text = this._ResourceManager.GetString("AfterDownload");
            this.doNothingToolStripMenuItem.Text = this._ResourceManager.GetString("DoNothing");
            this.closeRipperToolStripMenuItem.Text = this._ResourceManager.GetString("CloseRipper"); 

            this.settingsToolStripMenuItem.Text = this._ResourceManager.GetString("MenuHelp");
            this.helpToolStripMenuItem.Text = this._ResourceManager.GetString("MenuAbout");
        }

        /// <summary>
        /// Mains the form load.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void MainFormLoad(object sender, EventArgs e)
        {
            AppDomain.CurrentDomain.UnhandledException += this.UnhandledExceptionFunction;
            Application.ThreadException += this.ThreadExceptionFunction;

            this.LastWorkingTime = DateTime.Now;

            this.mrefCC = CacheController.Instance();
            this.mrefTM = ThreadManager.GetInstance();

            this.LoadSettings();

            this.SetWindow();

#if (!PGRIPPERX)
            string mbUpdate = this._ResourceManager.GetString("mbUpdate"), mbUpdate2 = this._ResourceManager.GetString("mbUpdate2");

            if (VersionCheck.UpdateAvailable() && File.Exists(Path.Combine(Application.StartupPath, "ICSharpCode.SharpZipLib.dll")))
            {
                DialogResult result = TopMostMessageBox.Show(
                    mbUpdate, mbUpdate2, MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (result == DialogResult.Yes)
                {
                    Clipboard.Clear();

                    Enabled = false;
                    WindowState = FormWindowState.Minimized;

                    trayIcon.BalloonTipIcon = ToolTipIcon.Info;
                    trayIcon.BalloonTipTitle = "Update in Progress";
                    trayIcon.BalloonTipText = "Currently upgrading to newest version.";
                    trayIcon.ShowBalloonTip(10);

                    AutoUpdater.TryUpdate();
                }
            }
#endif

            this.AutoLogin();

            if (!bCameThroughCorrectLogin)
            {
                Application.Exit();
            }
            else
            {
                this.CheckAccountMenu();
            }

            if (this.userSettings.SavePids)
            {
                this.LoadHistory();
            }

            // Hide Index Thread Checkbox if not RiP Forums
            if (!this.userSettings.CurrentForumUrl.Contains(@"vipergirls.to"))
            {
                mIsIndexChk.Visible = false;
            }

#if (!PGRIPPERX)
            trayIcon.Visible = true;
#endif
        }

        /// <summary>
        /// Add Forum Accounts to the Menu to allow switching.
        /// </summary>
        private void CheckAccountMenu()
        {
            this.accountsToolStripMenuItem.DropDownItems.Clear();

            var newAccountMenuItem = new ToolStripMenuItem
            {
                Text = this._ResourceManager.GetString("AddNewAccount"),
                Checked = false
            };

            newAccountMenuItem.Click += this.AddNewAccount_Click;

            this.accountsToolStripMenuItem.DropDownItems.Add(newAccountMenuItem);

            this.accountsToolStripMenuItem.DropDownItems.Add(new ToolStripSeparator());

            var deleteForumString = this._ResourceManager.GetString("DeleteAccount");

            foreach (var accounts in this.userSettings.ForumsAccount)
            {
                var forumName = accounts.ForumURL.Replace("http://", string.Empty);

                if (forumName.Contains("www."))
                {
                    forumName = forumName.Replace("www.", string.Empty);
                }

                if (forumName.EndsWith("/"))
                {
                    forumName = forumName.Remove(forumName.Length - 1);
                }

                var forumMenuItem = new ToolStripMenuItem
                    {
                        Tag = accounts.ForumURL,
                        Text = forumName, 
                        Checked = this.userSettings.CurrentForumUrl.Equals(accounts.ForumURL)
                    };

                forumMenuItem.Click += this.ForumMenuItem_Click;

                var forumDeleteMenuItem = new ToolStripMenuItem
                    {
                        Tag = accounts.ForumURL,
                        Text = deleteForumString
                    };

                forumDeleteMenuItem.Click += this.ForumMenuDeleteItem_Click;

                forumMenuItem.DropDownItems.Add(forumDeleteMenuItem);

                this.accountsToolStripMenuItem.DropDownItems.Add(forumMenuItem);
            }
        }

        /// <summary>
        /// Go To the Login Dialog to add a New Forum Account
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AddNewAccount_Click(object sender, EventArgs e)
        {
            this.Visible = false;

            Login frmLgn = new Login();
            frmLgn.ShowDialog(this);

            this.CheckAccountMenu();

            this.SwitchAccount(null);

            this.Visible = true;
        }

        /// <summary>
        /// Switch Login with Selected Account
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ForumMenuItem_Click(object sender, EventArgs e)
        {
            this.SwitchAccount(sender);
        }

        /// <summary>
        /// Delete the Forum Account
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ForumMenuDeleteItem_Click(object sender, EventArgs e)
        {
            var menuItem = (ToolStripMenuItem)sender;

            var deleteForumUrl = (string)menuItem.Tag;

            if (CacheController.Xform.userSettings.ForumsAccount.Any(item => item.ForumURL.Equals(deleteForumUrl)))
            {
                CacheController.Xform.userSettings.ForumsAccount.RemoveAll(item => item.ForumURL.Equals(deleteForumUrl));
            }

            Utility.SaveSettings(this.userSettings);

            this.CheckAccountMenu();

            // if account is current account switch to the first available account
            if (CacheController.Xform.userSettings.CurrentForumUrl.Equals(deleteForumUrl))
            {
                this.SwitchCurrentForumAccount(CacheController.Xform.userSettings.ForumsAccount.First().ForumURL);
            }
            else
            {
                this.SwitchAccount(null);
            }
        }

        /// <summary>
        /// Checks the URL if it matches the current forum account.
        /// </summary>
        /// <param name="forumUrl">The forum URL.</param>
        private void CheckUrlForumAccount(string forumUrl)
        {
            if (forumUrl.StartsWith(this.userSettings.CurrentForumUrl))
            {
                return;
            }

            this.SwitchCurrentForumAccount(forumUrl);
        }

        /// <summary>
        /// Switches the forum account.
        /// </summary>
        /// <param name="sender">The sender.</param>
        private void SwitchAccount(object sender)
        {
            // Kill all current downloads 
            /*if (mCurrentJob != null)
            {
                stopCurrentThreads.Enabled = false;
                this.stopingJob = true;

                ThreadManager.GetInstance().DismantleAllThreads();

                lvCurJob.Items.Clear();
                mCurrentJob = null;
                if (this.InvokeRequired)
                {
                    this.Invoke(
                        (MethodInvoker)delegate
                        {
                            StatusLabelImageC.Text = string.Empty;
                        });
                }
                else
                {
                    StatusLabelImageC.Text = string.Empty;
                }

                this.IdleRipper();
            }*/
            ////

            if (sender == null)
            {
                return;
            }

            var menuItem = (ToolStripMenuItem)sender;

            this.SwitchCurrentForumAccount((string)menuItem.Tag);
        }

        /// <summary>
        /// Switches the current forum account to the forum Url
        /// </summary>
        /// <param name="forumUrl">The forum URL.</param>
        private void SwitchCurrentForumAccount(string forumUrl)
        {
            ForumAccount currentAccount = null;

            foreach (var forumAccount in
                this.userSettings.ForumsAccount.Where(forumAccount => forumUrl.StartsWith(forumAccount.ForumURL)).Where(
                    forumAccount =>
                    !string.IsNullOrEmpty(forumAccount.UserName) && !string.IsNullOrEmpty(forumAccount.UserPassWord)))
            {
                currentAccount = forumAccount;
            }

            if (currentAccount != null)
            {
                this.userSettings.CurrentForumUrl = currentAccount.ForumURL;
                this.userSettings.CurrentUserName = currentAccount.UserName;

                if (currentAccount.GuestAccount)
                {
                    this.bCameThroughCorrectLogin = true;
                    this.CheckAccountMenu();
                }
                else
                {
                    var lgnMgr = new LoginManager(currentAccount.UserName, currentAccount.UserPassWord);

                    if (lgnMgr.DoLogin())
                    {
                        this.bCameThroughCorrectLogin = true;
                        this.CheckAccountMenu();
                    }
                    else
                    {
                        this.Visible = false;

                        Login frmLgn = new Login();
                        frmLgn.ShowDialog(this);

                        this.Visible = true;
                    }
                }
            }
            else
            {
                return;
            }

            var header = this._ResourceManager.GetString("ttlHeader");

#if (PGRIPPER)
            Text = string.Format(
                "PG-Ripper {0}.{1}.{2}{3}{4}",
                Assembly.GetExecutingAssembly().GetName().Version.Major.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Minor.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Build.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Revision.ToString("0"),
                string.Format("{0}{1}\" @ {2} ]", header, this.userSettings.CurrentUserName, this.userSettings.CurrentForumUrl));
#elif (PGRIPPERX)
                Text = string.Format("PG-Ripper X {0}.{1}.{2}{3}{4}", 
                    Assembly.GetExecutingAssembly().GetName().Version.Major.ToString("0"), 
                    Assembly.GetExecutingAssembly().GetName().Version.Minor.ToString("0"), 
                    Assembly.GetExecutingAssembly().GetName().Version.Build.ToString("0"), 
                    Assembly.GetExecutingAssembly().GetName().Version.Revision.ToString("0"),
                    string.Format("{0}{1}\" @ {2} ]", header, this.userSettings.CurrentUserName, this.userSettings.CurrentForumUrl));
#else
            Text = string.Format(
                "PG-Ripper {0}.{1}.{2}{3}{4}",
                Assembly.GetExecutingAssembly().GetName().Version.Major.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Minor.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Build.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Revision.ToString("0"),
                string.Format(
                    "{0}{1}\" @ {2} ]", header, this.userSettings.CurrentUserName, this.userSettings.CurrentForumUrl));
#endif
        }

        /// <summary>
        /// Auto login if User Credentials exists in Config file.
        /// if no Data show Login Form
        /// </summary>
        private void AutoLogin()
        {
            bool accountExists = false;
            var currentForumAccount = new ForumAccount();

            foreach (var forumAccount in this.userSettings.ForumsAccount.Where(
                    forumAccount =>
                        forumAccount.ForumURL.Equals(this.userSettings.CurrentForumUrl)).Where(
                        forumAccount => !string.IsNullOrEmpty(forumAccount.UserName) && !string.IsNullOrEmpty(forumAccount.UserPassWord)))
            {
                currentForumAccount = forumAccount;
                accountExists = true;
            }

            if (accountExists)
            {
                if (currentForumAccount.GuestAccount)
                {
                    this.bCameThroughCorrectLogin = true;
                }
                else
                {
                    LoginManager lgnMgr = new LoginManager(currentForumAccount.UserName, currentForumAccount.UserPassWord);

                    if (lgnMgr.DoLogin())
                    {
                        this.bCameThroughCorrectLogin = true;
                    }
                    else
                    {
                        Login frmLgn = new Login();
                        frmLgn.ShowDialog(this);

                        DialogResult result = TopMostMessageBox.Show(
                            this._ResourceManager.GetString("mbExit"),
                            this._ResourceManager.GetString("mbExitTtl"),
                            MessageBoxButtons.YesNo,
                            MessageBoxIcon.Question);

                        if (result == DialogResult.Yes)
                        {
                            Application.Exit();
                        }
                    }
                }
            }
            else
            {
                Login frmLgn = new Login();
                frmLgn.ShowDialog(this);

                Application.Exit();
            }
        }

        /// <summary>
        /// Starts Ripping
        /// </summary>
        /// <param name="sender">
        /// The sender object.
        /// </param>
        /// <param name="e">
        /// The Event Arguments.
        /// </param>
        private void MStartDownloadBtnClick(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(this.textBox1.Text) || this.parseActive)
            {
                return;
            }

            if (this.textBox1.Text.StartsWith("http"))
            {
                this.comboBox1.SelectedIndex = 2;
            }

            if (this.InvokeRequired)
            {
                this.Invoke((MethodInvoker)this.EnqueueJob);
            }
            else
            {
                this.EnqueueJob();
            }
        }

        /// <summary>
        /// Starts the Download when Pressing Enter in Textbox URL
        /// </summary>
        /// <param name="sender">
        /// The sender object.
        /// </param>
        /// <param name="e">
        /// The Key Press Event Arguments.
        /// </param>
        private void TextBox1KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar != '\r' || this.parseActive)
            {
                return;
            }

            e.Handled = true;

            if (this.InvokeRequired)
            {
                this.Invoke((MethodInvoker)this.EnqueueJob);
            }
            else
            {
                this.EnqueueJob();
            }
        }

        /// <summary>
        /// Enqueue Job
        /// </summary>
        private void EnqueueJob()
        {
            if (this.comboBox1.SelectedIndex.Equals(2))
            {
                // Encode URL
                this.textBox1.Text = Uri.EscapeUriString(this.textBox1.Text);

                this.CheckUrlForumAccount(this.textBox1.Text);
            }


            this.tmrPageUpdate.Enabled = true;

            if (!this.IsValidJob())
            {
                this.UnlockControls();
                return;
            }

            // Format Url
            var sHtmlUrl = UrlHandler.GetHtmlUrl(this.textBox1.Text, this.comboBox1.SelectedIndex);

            if (string.IsNullOrEmpty(sHtmlUrl))
            {
                this.UnlockControls();
                return;
            }

            // Check Post is Ripped?!
            if (this.userSettings.SavePids)
            {
                string sPostId = null;

                if (sHtmlUrl.Contains("showpost.php?p="))
                {
                    sPostId = sHtmlUrl.Substring(sHtmlUrl.IndexOf("showpost.php?p=") + 15);
                }
                else if (sHtmlUrl.Contains("#post"))
                {
                    sPostId = sHtmlUrl.Substring(sHtmlUrl.IndexOf("#post") + 5);
                }

                if (!string.IsNullOrEmpty(sPostId))
                {
                    if (sPostId.Contains("&postcount"))
                    {
                        sPostId.Remove(sHtmlUrl.IndexOf("&postcount"));
                    }

                    if (this.IsPostAlreadyRipped(sPostId))
                    {
                        DialogResult result = TopMostMessageBox.Show(
                            this._ResourceManager.GetString("mBAlready"), "Info", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                        if (result != DialogResult.Yes)
                        {
                            this.UnlockControls();
                            return;
                        }
                    }
                }
            }

            ///////////////////////////////////////////////
            this.LockControls();
            ///////////////////////////////////////////////

            if (mIsIndexChk.Checked)
            {
                // Parse Job as Index Thread
                this.EnqueueIndexThread(sHtmlUrl);
            }
            else
            {
                if (Utility.IsV4Forum(this.userSettings))
                {
                    if (!sHtmlUrl.Contains(@"#post") && !sHtmlUrl.Contains(@"showpost"))
                    {
                        this.EnqueueThreadToPost(sHtmlUrl);
                    }
                    else
                    {
                        this.EnqueueThreadOrPost(sHtmlUrl);
                    }
                }
                else
                {
                    if (sHtmlUrl.Contains(@"showthread.php"))
                    {
                        this.EnqueueThreadToPost(sHtmlUrl);
                    }
                    else
                    {
                        this.EnqueueThreadOrPost(sHtmlUrl);
                    }
                }
            }
        }

        /// <summary>
        /// Check if Job is OK
        /// </summary>
        /// <returns>Returns true or false</returns>
        private bool IsValidJob()
        {
            if (!Utility.IsNumeric(textBox1.Text) && comboBox1.SelectedIndex != 2)
            {
                TopMostMessageBox.Show(mTNumericMsg, "Info");

                return false;
            }

            if (string.IsNullOrEmpty(this.userSettings.DownloadFolder))
            {
                DialogResult result = TopMostMessageBox.Show(
                    "Please Set Up Download Folder before starting download",
                    "Info",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question);

                if (result == DialogResult.OK)
                {
                    this.UpdateDownloadFolder();
                }
            }

            return true;
        }

        /// <summary>
        /// Parse all Threads/Posts in the Index
        /// </summary>
        /// <param name="sHtmlUrl">The Url to the Thread/Post</param>
        private void EnqueueIndexThread(string sHtmlUrl)
        {
#if (!PGRIPPERX)
            if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                Environment.OSVersion.Version.Minor >= 1)
            {
                this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Indeterminate);
            }
#endif

            try
            {
                if (
                    string.IsNullOrEmpty(
                        Maintenance.GetInstance().GetRipPageTitle(Maintenance.GetInstance().GetPostPages(sHtmlUrl))))
                {
                    TopMostMessageBox.Show(sHtmlUrl.IndexOf("showthread.php") > 0 ? mNoThreadMsg : mNoPostMsg, "Info");

#if (!PGRIPPERX)
                    if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                        Environment.OSVersion.Version.Minor >= 1)
                    {
                        this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Normal);
                        this.windowsTaskbar.SetProgressValue(10, 100);
                    }
#endif
                    // Unlock Controls
                    this.UnlockControls();

                    return;
                }
            }
            catch (Exception)
            {
                TopMostMessageBox.Show(sHtmlUrl.IndexOf("showthread.php") > 0 ? mNoThreadMsg : mNoPostMsg, "Info");

#if (!PGRIPPERX)
                if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                    Environment.OSVersion.Version.Minor >= 1)
                {
                    this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Normal);
                    this.windowsTaskbar.SetProgressValue(10, 100);
                }
#endif
                // Unlock Controls
                this.UnlockControls();

                return;
            }

            mIsIndexChk.Checked = false;

            GetIdxsWorker.RunWorkerAsync(sHtmlUrl);

            while (GetIdxsWorker.IsBusy)
            {
                Application.DoEvents();
            }
        }

        /// <summary>
        /// Parse all Posts of a Thread
        /// </summary>
        /// <param name="sHtmlUrl">The Thread Url</param>
        private void EnqueueThreadToPost(string sHtmlUrl)
        {
#if (!PGRIPPERX)
            if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                Environment.OSVersion.Version.Minor >= 1)
            {
                this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Indeterminate);
            }
#endif

           try
            {
                if (
                    string.IsNullOrEmpty(
                        Maintenance.GetInstance().GetRipPageTitle(Maintenance.GetInstance().GetPostPages(sHtmlUrl))))
                {
                    TopMostMessageBox.Show(sHtmlUrl.IndexOf("showthread.php") > 0 ? mNoThreadMsg : mNoPostMsg, "Info");

#if (!PGRIPPERX)
                    if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                        Environment.OSVersion.Version.Minor >= 1)
                    {
                        this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Normal);
                        this.windowsTaskbar.SetProgressValue(10, 100);
                    }
#endif
                    // Unlock Controls
                    this.UnlockControls();

                    return;
                }
           }
            catch (Exception)
            {
                TopMostMessageBox.Show(sHtmlUrl.IndexOf("showthread.php") > 0 ? mNoThreadMsg : mNoPostMsg, "Info");

#if (!PGRIPPERX)
                if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                    Environment.OSVersion.Version.Minor >= 1)
                {
                    this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Normal);
                    this.windowsTaskbar.SetProgressValue(10, 100);
                }
#endif
                // Unlock Controls
                this.UnlockControls();

                return;
            }

            ///////
            while (GetPostsWorker.IsBusy)
            {
                Application.DoEvents();
            }

            this.GetPostsWorker.RunWorkerAsync(sHtmlUrl);
        }

        /// <summary>
        /// Parse a Thread or Post as Single Job
        /// </summary>
        /// <param name="sHtmlUrl">The Thread/Post Url</param>
        private void EnqueueThreadOrPost(string sHtmlUrl)
        {
            if (jobsList.Any(t => t.URL == sHtmlUrl))
            {
                TopMostMessageBox.Show(mAlreadyQueuedMsg, "Info");
                return;
            }

            JobInfo job = new JobInfo
                { URL = sHtmlUrl, HtmlPayLoad = Maintenance.GetInstance().GetPostPages(sHtmlUrl) };

            job.Title = Utility.ReplaceHexWithAscii(Maintenance.GetInstance().GetRipPageTitle(job.HtmlPayLoad));

            if (this.userSettings.AutoThank & Utility.IsV4Forum(this.userSettings))
            {
                job.SecurityToken = Maintenance.GetInstance().GetSecurityToken(job.HtmlPayLoad);
            }

            if (!sHtmlUrl.Contains(@"showthread") || sHtmlUrl.Contains(@"#post"))
            {
                job.PostTitle =
                    Utility.ReplaceHexWithAscii(
                        Maintenance.GetInstance().ExtractPostTitleFromHtml(job.HtmlPayLoad, sHtmlUrl));

                if (Utility.IsV4Forum(this.userSettings))
                {
                    job.ForumTitle = Maintenance.GetInstance().ExtractForumTitleFromHtml(job.URL, true);
                }
                else
                {
                    job.ForumTitle = Maintenance.GetInstance().ExtractForumTitleFromHtml(job.HtmlPayLoad, true);

                    job.ForumTitle =
                        job.ForumTitle.Substring(
                            job.ForumTitle.IndexOf(string.Format("{0} ", job.Title)) + job.Title.Length + 1);
                }
            }
            else
            {
                if (Utility.IsV4Forum(this.userSettings))
                {
                    job.ForumTitle = Maintenance.GetInstance().ExtractForumTitleFromHtml(job.URL, false);
                }
                else
                {
                    job.ForumTitle = Maintenance.GetInstance().ExtractForumTitleFromHtml(job.HtmlPayLoad, false);

                    job.ForumTitle =
                        job.ForumTitle.Substring(
                            job.ForumTitle.IndexOf(string.Format("{0} ", job.Title)) + job.Title.Length + 1);
                }
            }

            job.ImageList = Utility.ExtractImagesHtml(job.HtmlPayLoad, sHtmlUrl);
            job.ImageCount = job.ImageList.Count;

            if (job.ImageCount == 0)
            {
                // Unlock Controls
                this.UnlockControls();

                return;
            }

            if (string.IsNullOrEmpty(job.Title))
            {
                TopMostMessageBox.Show(sHtmlUrl.IndexOf("threadid=") > 0 ? mNoThreadMsg : mNoPostMsg, "Info");

                // Unlock Controls
                this.UnlockControls();

                return;
            }

            job.StorePath = this.GenerateStorePath(job);

            JobListAddDelegate newJob = this.JobListAdd;

            Invoke(newJob, new object[] { job });

            ///////////////////////////////////////////////
            this.UnlockControls();
            ///////////////////////////////////////////////
        }

        /// <summary>
        /// Pushes the "Thank You" Button for the user.
        /// </summary>
        /// <param name="sPostId">
        /// The s post id.
        /// </param>
        /// <param name="iICount">
        /// The i i count.
        /// </param>
        /// <param name="sSecurityToken">
        /// The s security token.
        /// </param>
        private void ProcessAutoThankYou(string sPostId, int iICount, string sSecurityToken)
        {
            if (!this.userSettings.AutoThank)
            {
                return;
            }

            if (iICount < this.userSettings.MinImageCount)
            {
                return;
            }

            SendThankYouDelegate lSendThankYouDel = this.SendThankYou;

            if (sPostId == null)
            {
                return;
            }

            string tyURL;
            if (this.userSettings.CurrentForumUrl.Contains(@"scanlover.com"))
            {
                tyURL = string.Format("{0}post_thanks.php?do=post_thanks_add&p={1}", this.userSettings.CurrentForumUrl, sPostId);
            }
            else if (Utility.IsV4Forum(this.userSettings))
            {
                tyURL = string.Format(
                    "{0}post_thanks.php?do=post_thanks_add&p={1}&securitytoken={2}",
                    this.userSettings.CurrentForumUrl,
                    sPostId,
                    sSecurityToken);
            }
            else
            {
                return;
            }

            this.Invoke(lSendThankYouDel, new object[] { tyURL });
        }

        /// <summary>
        /// This delegate enables asynchronous calls for automatically sending thank yous
        /// </summary>
        /// <param name="aUrl">A URL.</param>
        private delegate void SendThankYouDelegate(string aUrl);

        /// <summary>
        /// Sends the thank you.
        /// </summary>
        /// <param name="aUrl">A URL.</param>
        private void SendThankYou(string aUrl)
        {
            string tyURLRef = this.userSettings.CurrentForumUrl;

            HttpWebResponse lHttpWebResponse = null;
            Stream lHttpWebResponseStream = null;

            HttpWebRequest lHttpWebRequest = (HttpWebRequest)WebRequest.Create(aUrl);
            lHttpWebRequest.UserAgent =
                "Mozilla/5.0 (Windows; U; Windows NT 5.2; en-US; rv:1.7.10) Gecko/20050716 Firefox/1.0.6";
            lHttpWebRequest.Headers.Add("Accept-Language: en-us,en;q=0.5");
            lHttpWebRequest.Headers.Add("Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7");
            lHttpWebRequest.Headers.Add(string.Format("Cookie: {0}", CookieManager.GetInstance().GetCookieString()));
            lHttpWebRequest.Accept =
                "text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5";
            lHttpWebRequest.KeepAlive = true;
            lHttpWebRequest.Referer = tyURLRef;
            lHttpWebRequest.AllowAutoRedirect = false;
            lHttpWebRequest.Timeout = 1500;
            ///////////////////////////////////

            try
            {
                lHttpWebResponse = (HttpWebResponse)lHttpWebRequest.GetResponse();
                lHttpWebResponseStream = lHttpWebRequest.GetResponse().GetResponseStream();
            }
            finally
            {
                lHttpWebResponse.Close();
                lHttpWebResponseStream.Close();
            }
        }

        /// <summary>
        /// Parses an index and places all linked images in mDownloadsList.
        /// </summary>
        /// <param name="sHtmlUrl">The s HTML URL.</param>
        private void ThrdGetIndexes(string sHtmlUrl)
        {
            Indexes idxs = new Indexes();

            var pagecontent = Utility.IsV4Forum(this.userSettings)
                                      ? idxs.GetThreadPagesNew(sHtmlUrl)
                                      : idxs.GetThreadPages(sHtmlUrl);

            this.indexedTopicsList = idxs.ParseHtml(pagecontent, sHtmlUrl);
        }

        /// <summary>
        /// Get All Post of a Thread and Parse them as new Job
        /// </summary>
        /// <param name="htmlUrl">
        /// The html Url.
        /// </param>
        private void ThrdGetPosts(string htmlUrl)
        {
            ThreadToPost threads = new ThreadToPost();

            string pagecontent = Utility.IsV4Forum(this.userSettings)
                                      ? threads.GetThreadPagesNew(htmlUrl)
                                      : threads.GetThreadPages(htmlUrl);

            string forumTitle = Maintenance.GetInstance().ExtractForumTitleFromHtml(htmlUrl, false);
           
            List<ImageInfo> arlst = threads.ParseHtml(pagecontent);

            for (int po = 0; po < arlst.Count; po++)
            {
                var po1 = po;

                if (this.InvokeRequired)
                {
                    this.Invoke(
                        (MethodInvoker)delegate
                            {
                                this.StatusLabelInfo.Text = string.Format(
                                    "{0}{1}/{2}", this._ResourceManager.GetString("gbParse"), po1, arlst.Count);

                                this.StatusLabelInfo.ForeColor = Color.Green;
                            });
                }
                else
                {
                    this.StatusLabelInfo.Text = string.Format(
                       "{0}{1}/{2}", this._ResourceManager.GetString("gbParse"), po, arlst.Count);
                    this.StatusLabelInfo.ForeColor = Color.Green;
                }

                string postId = arlst[po].ImageUrl;

                //////////////////////////////////////////////////////////////////////////

                if (this.userSettings.SavePids && this.IsPostAlreadyRipped(postId))
                {
                    continue;
                }

                string newPostUrl = string.Format(
                    "{0}showpost.php?p={1}#post{1}", this.userSettings.CurrentForumUrl, postId);
                
                JobInfo jobInfoDouble = this.jobsList.Find(doubleJob => doubleJob.URL.Equals(newPostUrl));

                if (jobInfoDouble != null)
                {
                    continue;
                }

                if (this.currentJob != null)
                {
                    if (this.currentJob.URL.Equals(newPostUrl))
                    {
                        continue;
                    }
                }

                var newJob = new JobInfo
                                 {
                                     URL = newPostUrl,
                                     HtmlPayLoad = Maintenance.GetInstance().GetPostPages(newPostUrl)
                                 };

                if (string.IsNullOrEmpty(newJob.HtmlPayLoad))
                {
                    continue;
                }

                newJob.Title = Maintenance.GetInstance().GetRipPageTitle(newJob.HtmlPayLoad);

                if (this.userSettings.AutoThank & Utility.IsV4Forum(this.userSettings))
                {
                    newJob.SecurityToken = Maintenance.GetInstance().GetSecurityToken(newJob.HtmlPayLoad);
                }

                newJob.ForumTitle = Utility.IsV4Forum(this.userSettings)
                                     ? forumTitle
                                     : forumTitle.Substring(
                                         forumTitle.IndexOf(string.Format("{0} ", newJob.Title)) + newJob.Title.Length + 1);

                newJob.PostTitle = Maintenance.GetInstance().ExtractPostTitleFromHtml(newJob.HtmlPayLoad, newPostUrl);
                newJob.Title = Utility.ReplaceHexWithAscii(newJob.Title);

                newJob.ImageList = Utility.ExtractImagesHtml(newJob.HtmlPayLoad, postId);
                newJob.ImageCount = newJob.ImageList.Count;

                if (newJob.ImageCount == 0)
                {
                    continue;
                }

                newJob.StorePath = this.GenerateStorePath(newJob);

                JobListAddDelegate newJobDelegate = this.JobListAdd;

                this.Invoke(newJobDelegate, new object[] { newJob });

                //// JobListAdd(job);

                //////////////////////////////////////////////////////////////////////////
            }
        }

        /// <summary>
        /// Timer thats fires up the LogicCode
        /// </summary>
        /// <param name="sender">
        /// The sender object.
        /// </param>
        /// <param name="e">
        /// The Elapsed Event Arguments.
        /// </param>
        private void TmrPageUpdateElapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            if (this.working && !this.parseActive || this.jobsList.Count > 0 && !this.parseActive)
            {
                this.LogicCode();
            }

            if (!this.parseActive && this.ExtractUrls.Count > 0 && !this.GetPostsWorker.IsBusy && !this.GetIdxsWorker.IsBusy)
            {
                this.GetExtractUrls();
            }

            this.CheckClipboardData();
        }

        /// <summary>
        /// Over time, it turned into a nasty C-like code; where everything is
        /// in the same method...
        /// Its kind of an over kill and I need to rewrite this sorry excuse 
        /// for a 'scheduler'...
        /// </summary>
        private void LogicCode()
        {
            // Full HDD solution
            if (Delete && !this.fullDisc)
            {
                this.FullDisc();
            }

            if (mrefTM.GetThreadCount() > 0)
            {
                // If Joblist empty and the last Threads of Current Job are parsed
                if (this.currentJob == null && this.jobsList.Count.Equals(0) && !this.parseActive)
                {
                    this.StatusLabelInfo.Text = this._ResourceManager.GetString("StatusLabelInfo");
                    StatusLabelInfo.ForeColor = Color.Red;

                    this.groupBox5.Text = string.Format("{0} (-):", this._ResourceManager.GetString("lblRippingQue"));
                }
            }
            else
            {
                // Check if Last Downloadfolder is Empty
                if (!string.IsNullOrEmpty(this.lastDownFolder))
                {
                    CheckCurJobFolder(this.lastDownFolder);
                }

                if (!this.userSettings.CurrentlyPauseThreads)
                {
                    lvCurJob.Items.Clear();
                    if (this.InvokeRequired)
                    {
                        this.Invoke(
                            (MethodInvoker)delegate
                            {
                                StatusLabelImageC.Text = string.Empty;
                            });
                    }
                    else
                    {
                        StatusLabelImageC.Text = string.Empty;
                    }

                    if (this.currentJob == null)
                    {
                        if (this.jobsList.Count.Equals(0))
                        {
                            this.IdleRipper(true);
                        }
                        else
                        {
#if (!PGRIPPERX)
                            if (Environment.OSVersion.Platform == PlatformID.Win32NT
                                && Environment.OSVersion.Version.Major >= 6 && Environment.OSVersion.Version.Minor >= 1)
                            {
                                this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Indeterminate);
                            }
#endif

                            // STARTING TO PROCESS NEXT THREAD IN DOWNLOAD JOBS LIST
                            this.ProcessNextJob();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// STARTING TO PROCESS NEXT THREAD IN DOWNLOAD JOBS LIST
        /// </summary>
        private void ProcessNextJob()
        {
            this.working = true;

            deleteJob.Enabled = true;
            stopCurrentThreads.Enabled = true;

            if (jobsList.Count == 0)
            {
                return;
            }

            currentJob = jobsList[0];

            this.JobListRemove(0);

            string bSystemExtr = this._ResourceManager.GetString("bSystemExtr");

            this.ParseJob();

            // NO IMAGES TO PROCESS SO ABANDON CURRENT THREAD
            if (mImagesList == null || mImagesList.Count <= 0)
            {
                currentJob = null;
                deleteJob.Enabled = true;
                stopCurrentThreads.Enabled = true;
                return;
            }

            this.groupBox2.Text = string.Format("{0}...", this._ResourceManager.GetString("gbCurrentlyExtract"));

            if (currentJob.Title.Equals(currentJob.PostTitle))
            {
                Text = string.Format(
                    "{0}: {1} - x{2}", this._ResourceManager.GetString("gbCurrentlyExtract"), this.currentJob.Title, this.mImagesList.Count);
            }
            else
            {
                Text = string.Format(
                    "{0}: {1} - {2} - x{3}",
                    this._ResourceManager.GetString("gbCurrentlyExtract"),
                    currentJob.Title,
                    currentJob.PostTitle,
                    mImagesList.Count);
            }

            lvCurJob.Columns[0].Text = string.Format("{0} - x{1}", currentJob.PostTitle, mImagesList.Count);

#if (!PGRIPPERX)
            try
            {
                if (this.userSettings.ShowPopUps)
                {
                    this.trayIcon.Text = this._ResourceManager.GetString("gbCurrentlyExtract") + this.currentJob.Title;
                    trayIcon.BalloonTipIcon = ToolTipIcon.Info;
                    this.trayIcon.BalloonTipTitle = this._ResourceManager.GetString("gbCurrentlyExtract");
                    trayIcon.BalloonTipText = currentJob.Title;
                    trayIcon.ShowBalloonTip(10);
                }
            }
            catch (Exception)
            {
                if (this.userSettings.ShowPopUps)
                {
                    trayIcon.Text = bSystemExtr;
                    trayIcon.BalloonTipTitle = bSystemExtr;
                    trayIcon.BalloonTipText = bSystemExtr;
                    trayIcon.ShowBalloonTip(10);
                }
            }
#endif

            try
            {
                progressBar1.Maximum = mImagesList.Count;
            }
            catch (Exception)
            {
                progressBar1.Maximum = 10000;
            }

            this.ProcessCurImgLst();
        }

        /// <summary>
        /// Processing the Images list of the Current Job
        /// </summary>
        private void ProcessCurImgLst()
        {
            stopCurrentThreads.Enabled = true;
            this.stopingJob = false;
            this.working = true;

            this.lastDownFolder = null;

            ThreadManager lTdm = ThreadManager.GetInstance();

            this.lastDownFolder = currentJob.StorePath;

            if (mImagesList.Count > 0)
            {
                string tiImagesRemain = this._ResourceManager.GetString("tiImagesRemain");

                ////////////////
                lvCurJob.Items.Clear();
                if (this.InvokeRequired)
                {
                    this.Invoke(
                        (MethodInvoker)delegate
                        {
                            StatusLabelImageC.Text = string.Empty;
                        });
                }
                else
                {
                    StatusLabelImageC.Text = string.Empty;
                }

                for (int i = 0; i < mImagesList.Count; i++)
                {
                    lvCurJob.Items.Add(
                        string.Format("{0}/{1} - {2}", i + 1, mImagesList.Count, mImagesList[i].ImageUrl),
                        mImagesList[i].ImageUrl);
                }
                ///////////////////

                progressBar1.Maximum = mImagesList.Count;

                for (int i = 0; i < mImagesList.Count; i++)
                {
                    if (this.stopingJob || this.ripperClosing)
                    {
                        break;
                    }

#if (!PGRIPPERX)
                    this.trayIcon.Text = string.Format(tiImagesRemain, this.mImagesList.Count - i, i * 100 / this.mImagesList.Count);

                    if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                        Environment.OSVersion.Version.Minor >= 1)
                    {
                        this.windowsTaskbar.SetProgressValue(i, mImagesList.Count);
                    }
#endif
                    while (!lTdm.IsSystemReadyForNewThread())
                    {
                        Application.DoEvents();
                    }

                    if (!this.ripperClosing)
                    {
                        if (this.progressBar1 != null)
                        {
                            this.progressBar1.Value = i;
                        }

                        this.StatusLabelImageC.Text = string.Format(
                            tiImagesRemain, mImagesList.Count - i, i * 100 / mImagesList.Count);
                    }

                    try
                    {
                        // Attempts to download an image when the URL is longer than http://
                        if (this.mImagesList[i].ImageUrl.Length > 6)
                        {
                            if (!(i > this.lvCurJob.Items.Count))
                            {
                                this.lvCurJob.Items[i].Selected = true;
                                this.lvCurJob.EnsureVisible(i);
                            }

                            CacheController.Instance()
                                .DownloadImage(
                                    this.mImagesList[i].ImageUrl,
                                    this.mImagesList[i].ThumbnailUrl,
                                    this.currentJob.StorePath,
                                    !string.IsNullOrEmpty(this.currentJob.PostTitle)
                                        ? this.currentJob.PostTitle
                                        : this.currentJob.Title,
                                    i + 1);
                        }
                    }
                    catch (ArgumentOutOfRangeException)
                    {
                        if (this.stopingJob || this.ripperClosing)
                        {
                            break;
                        }
                    }
                    catch (NullReferenceException)
                    {
                        if (this.stopingJob || this.ripperClosing)
                        {
                            break;
                        }
                    }

                    if (i > this.lvCurJob.Items.Count)
                    {
                        continue;
                    }

                    if (this.InvokeRequired)
                    {
                        this.Invoke((MethodInvoker)this.ShowLastPic);
                    }
                    else
                    {
                        this.ShowLastPic();
                    }

                    if (!this.ripperClosing)
                    {
                        this.lvCurJob.Items[i].ForeColor = Color.Green;
                    }
                }

                // FINISED A THREAD/POST DOWNLOAD JOB
                currentJob = null;

                if (!string.IsNullOrEmpty(this.lastDownFolder))
                {
                    CheckCurJobFolder(this.lastDownFolder);
                }

                if (jobsList.Count > 0)
                {
#if (!PGRIPPERX)
                    if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                        Environment.OSVersion.Version.Minor >= 1)
                    {
                        this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Indeterminate);
                    }
#endif
                    // STARTING TO PROCESS NEXT THREAD IN DOWNLOAD JOBS LIST
                    this.ProcessNextJob();
                }
            }
            else
            {
                // NO IMAGES TO PROCESS SO ABANDON CURRENT THREAD
                lvCurJob.Items.Clear();
                currentJob = null;
                if (this.InvokeRequired)
                {
                    this.Invoke(
                        (MethodInvoker)delegate
                        {
                            StatusLabelImageC.Text = string.Empty;
                        });
                }
                else
                {
                    StatusLabelImageC.Text = string.Empty;
                }
            }
        }

        private Image imgLastPic;

        /// <summary>
        /// Shows the last Downloaded Image
        /// </summary>
        private void ShowLastPic()
        {
            if (!this.userSettings.ShowLastDownloaded)
            {
                return;
            }

            if (this.pictureBox1.Image != null)
            {
                this.pictureBox1.Image.Dispose();
                this.pictureBox1.Image = null;
            }

            if (this.pictureBox1.BackgroundImage != null)
            {
                this.pictureBox1.BackgroundImage.Dispose();
                this.pictureBox1.BackgroundImage = null;
            }
            ///////////////////////////

            this.sLastPic = this.mrefCC.LastPic;

            if (!File.Exists(this.sLastPic))
            {
                return;
            }

            try
            {
                var fileInfo = new FileInfo(this.sLastPic);

                // skip Images above 9 MB
                if (fileInfo.Length >= 9437184)
                {
                    return;
                }

                this.pictureBox1.Visible = true;

                if (this.sLastPic.EndsWith(".gif"))
                {
                    this.pictureBox1.BackgroundImage = Image.FromFile(this.sLastPic);

                    this.pictureBox1.Update();
                }
                else
                {
                    // This statement causes file locking until the
                    // process exits unless cleared when not visible
                    //this.imgLastPic = Image.FromFile(this.sLastPic);

                    using (FileStream stream = new FileStream(this.sLastPic, FileMode.Open, FileAccess.Read))
                    {
                        this.imgLastPic = Image.FromStream(stream);
                    }

                    this.pictureBox1.Image = this.imgLastPic;
                    this.pictureBox1.Update();
                }
            }
            catch (Exception)
            {
                this.pictureBox1.Image.Dispose();
                this.pictureBox1.Image = null;
            }
        }

        /// <summary>
        /// Adds new Job to JobList and ListView
        /// </summary>
        /// <param name="job">The New Job</param>
        private delegate void JobListAddDelegate(JobInfo job);

        /// <summary>
        /// Adds new Job to JobList and ListView
        /// </summary>
        /// <param name="job">The new Job</param>
        private void JobListAdd(JobInfo job)
        {
            this.working = true;

            jobsList.Add(job);

            ListViewItem ijobJob = new ListViewItem { Text = job.Title };

            ijobJob.SubItems.Add(job.PostTitle);
            ijobJob.SubItems.Add(job.ImageCount.ToString());

            listViewJobList.Items.AddRange(new[] { ijobJob });

            this.groupBox5.Text = string.Format("{0} ({1}):", this._ResourceManager.GetString("lblRippingQue"), this.jobsList.Count);
        }

        /// <summary>
        /// Removes a Job from the Joblist and ListView
        /// </summary>
        /// <param name="iJobIndex">The Index of the Job inside the Joblist</param>
        private void JobListRemove(int iJobIndex)
        {
            jobsList.RemoveAt(iJobIndex);

            listViewJobList.Items.RemoveAt(iJobIndex);

            this.groupBox5.Text = string.Format("{0} ({1}):", this._ResourceManager.GetString("lblRippingQue"), this.jobsList.Count);
        }

        /// <summary>
        /// Idles the ripper.
        /// </summary>
        /// <param name="endingAllRip">Indicates that Ripper is Finishing the last Threads of all Jobs</param>
        private void IdleRipper(bool endingAllRip = false)
        {
#if (!PGRIPPERX)
            if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                Environment.OSVersion.Version.Minor >= 1)
            {
                this.windowsTaskbar.SetProgressState(TaskbarProgressBarState.Normal);
                this.windowsTaskbar.SetProgressValue(10, 100);
            }

            string btleExit = this._ResourceManager.GetString("btleExit"), btexExit = this._ResourceManager.GetString("btexExit");

            if (endingAllRip && this.userSettings.ShowCompletePopUp)
            {
                this.trayIcon.BalloonTipIcon = ToolTipIcon.Info;
                this.trayIcon.BalloonTipTitle = btleExit;
                this.trayIcon.BalloonTipText = btexExit;
                this.trayIcon.ShowBalloonTip(10);
            }
#endif
            if (endingAllRip && this.userSettings.AfterDownloads.Equals(1))
            {
                this.Close();
            }
            else
            {
                this.lvCurJob.Items.Clear();

                this.stopCurrentThreads.Enabled = true;
                this.stopingJob = false;

                if (this.InvokeRequired)
                {
                    this.Invoke(
                        (MethodInvoker)delegate
                            {
                                StatusLabelImageC.Text = string.Empty;
                            });
                }
                else
                {
                    StatusLabelImageC.Text = string.Empty;
                }

                textBox1.Text = string.Empty;

                progressBar1.Value = 0;

                this.parseActive = false;

                string ttlHeader = this._ResourceManager.GetString("ttlHeader");

                this.groupBox2.Text = this._ResourceManager.GetString("gbCurrentlyIdle");
                this.StatusLabelInfo.Text = this._ResourceManager.GetString("gbCurrentlyIdle");
                StatusLabelInfo.ForeColor = Color.Gray;

                lvCurJob.Columns[0].Text = "  ";

#if (PGRIPPER)
            this.Text = string.Format(
                "PG-Ripper {0}.{1}.{2}{3}{4}",
                Assembly.GetExecutingAssembly().GetName().Version.Major.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Minor.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Build.ToString("0"),
                Assembly.GetExecutingAssembly().GetName().Version.Revision.ToString("0"),
                string.Format("{0}{1}\" @ {2} ]", ttlHeader, this.userSettings.CurrentUserName, this.userSettings.CurrentForumUrl));

            trayIcon.Text = "Right click for context menu";
#elif (PGRIPPERX)
                this.Text = string.Format("PG-Ripper X {0}.{1}.{2}{3}{4}", 
                    Assembly.GetExecutingAssembly().GetName().Version.Major.ToString("0"), 
                    Assembly.GetExecutingAssembly().GetName().Version.Minor.ToString("0"), 
                    Assembly.GetExecutingAssembly().GetName().Version.Build.ToString("0"), 
                    Assembly.GetExecutingAssembly().GetName().Version.Revision.ToString("0"),
                    string.Format("{0}{1}\" @ {2} ]", ttlHeader, this.userSettings.CurrentUserName, this.userSettings.CurrentForumUrl));
#else
                this.Text = string.Format(
                    "PG-Ripper {0}.{1}.{2}{3}{4}",
                    Assembly.GetExecutingAssembly().GetName().Version.Major.ToString("0"),
                    Assembly.GetExecutingAssembly().GetName().Version.Minor.ToString("0"),
                    Assembly.GetExecutingAssembly().GetName().Version.Build.ToString("0"),
                    Assembly.GetExecutingAssembly().GetName().Version.Revision.ToString("0"),
                    string.Format(
                        "{0}{1}\" @ {2} ]",
                        ttlHeader,
                        this.userSettings.CurrentUserName,
                        this.userSettings.CurrentForumUrl));

                trayIcon.Text = "Right click for context menu";
#endif
                // Since no picbox image will be visible until another job is queued,
                // reclaim resources used by any previous image in the picturebox
                // This prevents file locking of downloaded images until the process exits
                /*if (pictureBox1.Image != null)
                {
                    pictureBox1.Image.Dispose();
                    pictureBox1.Image = null;
                }*/

                if (this.imgLastPic != null)
                {
                    this.imgLastPic.Dispose();
                    this.imgLastPic = null;
                }

                if (pictureBox1.Image != null)
                {
                    pictureBox1.Image.Dispose();
                    pictureBox1.Image = null;
                }

                if (pictureBox1.BackgroundImage != null)
                {
                    pictureBox1.BackgroundImage.Dispose();
                    pictureBox1.BackgroundImage = null;
                }

                // Hide any image last displayed in the picturebox
                pictureBox1.Visible = false;

                deleteJob.Enabled = false;
                stopCurrentThreads.Enabled = false;

                this.working = false;
            }
        }

        /// <summary>
        /// Full HDD solution
        /// </summary>
        private void FullDisc()
        {
            this.working = false;
            this.fullDisc = true;

            pauseCurrentThreads.Text = "Resume Download(s)";
            ThreadManager.GetInstance().HoldAllThreads();

            StatusLabelInfo.Text = DeleteMessage;
            StatusLabelInfo.ForeColor = Color.Red;

            TopMostMessageBox.Show(
                string.Format(
                    "Please change your download location, then press \"Resume Download\", because {0}", DeleteMessage),
                "Warning");

            JobListAddDelegate updateJob = this.JobListAdd;

            Invoke(updateJob, new object[] { currentJob });

            lvCurJob.Items.Clear();
            if (this.InvokeRequired)
            {
                this.Invoke(
                    (MethodInvoker)delegate
                    {
                        StatusLabelImageC.Text = string.Empty;
                    });
            }
            else
            {
                StatusLabelImageC.Text = string.Empty;
            }

            currentJob = null;

            this.UpdateDownloadFolder();

            for (int i = 0; i != jobsList.Count; i++)
            {
                /* JobInfo updatedJob = new JobInfo
                                          {
                                              iImageCount = mJobsList[i].iImageCount,
                                              sPostTitle = mJobsList[i].sPostTitle,
                                              sStorePath = sDownloadFolder
                                          };

                 if (SubDirs)
                 {
                     if (comboBox1.SelectedIndex == 0)
                     {
                         updatedJob.sStorePath = Path.Combine(sDownloadFolder, mJobsList[i].sTitle);
                     }
                     if (comboBox1.SelectedIndex == 1 || comboBox1.SelectedIndex == 2)
                     {
                         updatedJob.sStorePath = Path.Combine(sDownloadFolder, mJobsList[i].sTitle + Path.DirectorySeparatorChar + mJobsList[i].sPostTitle);
                     }
                 }

                 updatedJob.sTitle = mJobsList[i].sTitle;
                 updatedJob.sURL = mJobsList[i].sURL;
                 updatedJob.sXMLPayLoad = mJobsList[i].sXMLPayLoad;

                 JobListRemove(i);

                 mJobsList.Insert(i, updatedJob);*/

                jobsList[i].StorePath = this.userSettings.DownloadFolder;

                if (!this.userSettings.SubDirs)
                {
                    continue;
                }

                if (comboBox1.SelectedIndex == 0)
                {
                    jobsList[i].StorePath = Path.Combine(this.userSettings.DownloadFolder, jobsList[i].Title);
                }

                if (comboBox1.SelectedIndex == 1 || comboBox1.SelectedIndex == 2)
                {
                    jobsList[i].StorePath = Path.Combine(
                        this.userSettings.DownloadFolder,
                        jobsList[i].Title + Path.DirectorySeparatorChar + jobsList[i].PostTitle);
                }
            }

            Delete = false;
            this.fullDisc = false;
        }

        /// <summary>
        /// Parses the job.
        /// </summary>
        private void ParseJob()
        {
            string sPostIdStart = null;

            if (currentJob.URL.Contains("showpost.php"))
            {
                sPostIdStart = "showpost.php?p=";
            }
            else if (currentJob.URL.Contains("#post"))
            {
                sPostIdStart = "#post";
            }

            string postId = currentJob.URL.Substring(currentJob.URL.IndexOf(sPostIdStart) + sPostIdStart.Length);

            if (postId.Contains(@"postcount"))
            {
                postId = Regex.Replace(postId, postId.Substring(postId.IndexOf("postcount=") - 1), string.Empty);
            }

            mImagesList = currentJob.ImageList;

            this.ProcessAutoThankYou(postId, this.mImagesList.Count, this.currentJob.SecurityToken);
        }

        /// <summary>
        /// Deletes the Selected Jobs
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DeleteJobClick(object sender, EventArgs e)
        {
            try
            {
                if (this.listViewJobList.SelectedItems.Count <= 0)
                {
                    return;
                }

                this.jobsList.RemoveRange(this.listViewJobList.SelectedItems[0].Index, this.listViewJobList.SelectedIndices.Count);

                foreach (ListViewItem deleteItem in this.listViewJobList.SelectedItems)
                {
                    this.listViewJobList.Items.Remove(deleteItem);
                }
            }
            catch (Exception)
            {
                this.listViewJobList.Items.Clear();

                for (int i = 0; i < this.jobsList.Count; i++)
                {
                    this.StatusLabelInfo.Text = string.Format("{0}{1}/{2}", this._ResourceManager.GetString("gbParse2"), i, this.jobsList.Count);
                    this.StatusLabelInfo.ForeColor = Color.Green;

                    var jobItem = new ListViewItem(this.jobsList[i].Title, 0);

                    jobItem.SubItems.Add(this.jobsList[i].PostTitle);
                    jobItem.SubItems.Add(this.jobsList[i].ImageCount.ToString());

                    this.listViewJobList.Items.AddRange(new[] { jobItem });
                }
            }
            finally
            {
                this.groupBox5.Text = string.Format("{0} ({1}):", this._ResourceManager.GetString("lblRippingQue"), this.jobsList.Count);
            }
        }

        /// <summary>
        /// Kill and Deletes the Current Job
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void StopCurrentThreadsClick(object sender, EventArgs e)
        {
            if (currentJob == null)
            {
                return;
            }

            stopCurrentThreads.Enabled = false;
            this.stopingJob = true;

            ThreadManager.GetInstance().DismantleAllThreads();

            string sLastJobFolder = currentJob.StorePath;

            lvCurJob.Items.Clear();
            currentJob = null;
            if (this.InvokeRequired)
            {
                this.Invoke(
                    (MethodInvoker)delegate
                    {
                        StatusLabelImageC.Text = string.Empty;
                    });
            }
            else
            {
                StatusLabelImageC.Text = string.Empty;
            }

            CheckCurJobFolder(sLastJobFolder);
        }

        /// <summary>
        /// Pause/Resumes Downloading
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void PauseCurrentThreadsClick(object sender, EventArgs e)
        {
            switch (this.pauseCurrentThreads.Text)
            {
                case "Pause Download(s)":
                    this.pauseCurrentThreads.Text = "Resume Download(s)";
                    this.userSettings.CurrentlyPauseThreads = true;
                    ThreadManager.GetInstance().HoldAllThreads();
                    this.pauseCurrentThreads.Image = Languages.english.play;
                    break;
                case "(Re)Start Download(s)":
                    if (this.InvokeRequired)
                    {
                        this.Invoke(
                            (MethodInvoker)delegate
                            {
                                this.StatusLabelImageC.Text = string.Empty;
                            });
                    }
                    else
                    {
                        StatusLabelImageC.Text = string.Empty;
                    }

                    this.userSettings.CurrentlyPauseThreads = false;
                    this.deleteJob.Enabled = true;
                    this.stopCurrentThreads.Enabled = true;
                    this.pauseCurrentThreads.Text = "Pause Download(s)";
                    this.pauseCurrentThreads.Image = Languages.english.pause;
                    break;
                case "Resume Download(s)":
                    if (this.InvokeRequired)
                    {
                        this.Invoke(
                            (MethodInvoker)delegate
                            {
                                this.StatusLabelImageC.Text = string.Empty;
                            });
                    }
                    else
                    {
                        StatusLabelImageC.Text = string.Empty;
                    }

                    this.userSettings.CurrentlyPauseThreads = false;
                    this.deleteJob.Enabled = true;
                    this.stopCurrentThreads.Enabled = true;
                    this.pauseCurrentThreads.Text = "Pause Download(s)";
                    
                    if (this.InvokeRequired)
                    {
                        this.Invoke(
                            (MethodInvoker)(() => ThreadManager.GetInstance().ResumeAllThreads()));
                    }
                    else
                    {
                        ThreadManager.GetInstance().ResumeAllThreads();
                    }
                    
                    this.pauseCurrentThreads.Image = Languages.english.pause;
                    break;
            }

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Comboes the box1 selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ComboBox1SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBox1.SelectedIndex)
            {
                case 0:
                    this.userSettings.DownloadOptions = "0";
                    break;
                case 1:
                    this.userSettings.DownloadOptions = "1";
                    break;
                case 2:
                    this.userSettings.DownloadOptions = "2";
                    break;
            }

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Mains the form resize.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void MainFormResize(object sender, EventArgs e)
        {
#if (!PGRIPPERX)
            if (WindowState == FormWindowState.Minimized)
            {
                this.Hide();
                this.isHiddenInTray = true;

                this.trayIcon.MouseDoubleClick -= this.HideClick;
                this.trayIcon.MouseDoubleClick += this.ShowClick;

                trayMenu.MenuItems.RemoveAt(0);
                MenuItem show = new MenuItem("Show PG-Ripper", this.ShowClick);
                trayMenu.MenuItems.Add(0, show);

                if (this.userSettings.ShowPopUps)
                {
                    trayIcon.BalloonTipIcon = ToolTipIcon.Warning;
                    trayIcon.BalloonTipTitle = "Hidden in Tray";
                    trayIcon.BalloonTipText = "PG-Ripper is hidden in the Tray";
                    trayIcon.ShowBalloonTip(10);
                }
            }
#endif
            lvCurJob.Columns[0].Width = lvCurJob.Width - 22;
        }

        /// <summary>
        /// Exits the tool strip menu item click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ExitToolStripMenuItemClick(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Save all Jobs
        /// </summary>
        public void SaveOnExit()
        {
            if (this.currentJob == null && this.jobsList.Count <= 0)
            {
                return;
            }

            // Save Current Job to quere List
            if (this.currentJob != null)
            {
                this.ripperClosing = true;

                ThreadManager.GetInstance().DismantleAllThreads();

                this.jobsList.Add(this.currentJob);

                this.currentJob = null;
            }

            var serializer = new XmlSerializer(typeof(List<JobInfo>));
            var textWriter = new StreamWriter(Path.Combine(Application.StartupPath, "jobs.xml"));

            serializer.Serialize(textWriter, this.jobsList);
            textWriter.Close();

            // If Pause
            if (this.pauseCurrentThreads.Text != "Resume Download")
            {
                return;
            }

            this.userSettings.CurrentlyPauseThreads = true;
            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Loads the Saved PostsIDs into Array
        /// </summary>
        private void LoadHistory()
        {
            string sFile = Path.Combine(Application.StartupPath, "rippedPostHistory.txt");

            FileStream file = new FileStream(sFile, FileMode.OpenOrCreate, FileAccess.Read);
            StreamReader sr = new StreamReader(file);
            string srRead = sr.ReadToEnd();

            if (srRead.Contains("\r\n"))
            {
                srRead = srRead.Replace("\r\n", string.Empty);
            }

            sr.Close();
            file.Close();

            string[] sPostIDs = srRead.Split(';');

            this.sRippedPosts.Clear();

            foreach (string sSavedId in sPostIDs.Where(sSavedId => !string.IsNullOrEmpty(sSavedId)))
            {
                this.sRippedPosts.Add(
                    sSavedId.Contains("&postcount") ? sSavedId.Remove(sSavedId.IndexOf("&postcount")) : sSavedId);
            }

            /*foreach (string sSavedId in sPostIDs)
            {
                if (sSavedId != "")
                {
                    if (sSavedId.Contains("&postcount"))
                    {
                        sRippedPosts.Add(sSavedId.Remove(sSavedId.IndexOf("&postcount")));
                    }
                    else
                    {
                        sRippedPosts.Add(sSavedId);
                    }
                }
            }*/
        }

        /// <summary>
        /// Save the Saved PostsIDs from Array to Textfile
        /// </summary>
        private void SaveHistory()
        {
            string sFile = Path.Combine(Application.StartupPath, "rippedPostHistory.txt");

            FileStream file = new FileStream(sFile, FileMode.Create);
            StreamWriter sw = new StreamWriter(file);

            foreach (string sSavedId in this.sRippedPosts)
            {
                sw.Write("{0};", sSavedId);
            }

            sw.Close();
            file.Close();
        }

        /// <summary>
        /// Check if Post is already Ripped
        /// </summary>
        /// <param name="sPostId">The PostID to Check</param>
        /// <returns>Returns true or false</returns>
        private bool IsPostAlreadyRipped(string sPostId)
        {
            bool bCheck = false;

            if (!Utility.IsNumeric(sPostId))
            {
                return false;
            }

            try
            {
                if (this.sRippedPosts.Contains(sPostId))
                {
                    bCheck = true;
                }
                else
                {
                    this.sRippedPosts.Add(sPostId);
                }
            }
            catch (Exception)
            {
                bCheck = false;
            }

            return bCheck;
        }

        /// <summary>
        /// Open the Options Dialog
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SettingsToolStripMenuItem1Click(object sender, EventArgs e)
        {
            if (this.userSettings.SavePids)
            {
                this.SaveHistory();
            }

            var oForm = new Options();
            oForm.ShowDialog();

            this.LoadSettings();

            if (this.userSettings.SavePids)
            {
                this.LoadHistory();
            }
        }

        /// <summary>
        /// Shows the About Dialog
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void HelpToolStripMenuItemClick(object sender, EventArgs e)
        {
            About aForm = new About();
            aForm.ShowDialog();
        }

        /// <summary>
        /// Update the Download Folder
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void BrowsFolderBtnClick(object sender, EventArgs e)
        {
            this.UpdateDownloadFolder();
        }

        /// <summary>
        /// Changes the download Folder.
        /// </summary>
        public void UpdateDownloadFolder()
        {
            if (this.IsBrowserOpen)
            {
                return;
            }

            this.IsBrowserOpen = true;

            if (this.dfolderBrowserDialog.ShowDialog(this) != DialogResult.OK)
            {
                return;
            }

            this.DownloadFolder.Text = this.dfolderBrowserDialog.SelectedPath;

            this.userSettings.DownloadFolder = this.DownloadFolder.Text;

            Utility.SaveSettings(this.userSettings);

            this.IsBrowserOpen = false;
        }

        /// <summary>
        /// Set Window Position and Size
        /// </summary>
        private void SetWindow()
        {
            try
            {
                Left = this.userSettings.WindowLeft;
                Top = this.userSettings.WindowTop;
            }
            catch (Exception)
            {
                StartPosition = FormStartPosition.CenterScreen;
            }

            Width = this.userSettings.WindowWidth;
            Height = this.userSettings.WindowHeight;
        }

        /// <summary>
        /// Mains the form form closing.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void MainFormFormClosing(object sender, FormClosingEventArgs e)
        {
            this.SaveOnExit();

            if (this.userSettings.SavePids)
            {
                this.SaveHistory();
            }

            if (this.WindowState == FormWindowState.Minimized || this.isHiddenInTray)
            {
                return;
            }

            this.userSettings.WindowLeft = this.Left;
            this.userSettings.WindowTop = this.Top;
            this.userSettings.WindowWidth = this.Width;
            this.userSettings.WindowHeight = this.Height;

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Check Windows Clip board for URL's to Rip
        /// </summary>
        private void CheckClipboardData()
        {
            if (!this.userSettings.ClipBWatch)
            {
                return;
            }

            try
            {
                var clipboardText = Clipboard.GetText();

                if (string.IsNullOrEmpty(clipboardText))
                {
                    return;
                }

                var clipBoardURLTemp = clipboardText;

                var clipBoardUrLs = clipBoardURLTemp.Split(new[] { '\n' });

                foreach (var clipBoardURL in clipBoardUrLs.Where(clipBoardURL => this.userSettings.ForumsAccount.Any(
                    forumAccount => clipBoardURL.StartsWith(forumAccount.ForumURL))))
                {
                    this.CheckUrlForumAccount(this.textBox1.Text);

                    string sClipBoardURLNew = clipBoardURL;

                    if (sClipBoardURLNew.Contains("\r"))
                    {
                        sClipBoardURLNew = sClipBoardURLNew.Replace("\r", string.Empty);
                    }

                    if (clipBoardURL.Contains(@"&postcount=") || clipBoardURL.Contains(@"&page="))
                    {
                        sClipBoardURLNew = Regex.Replace(
                            sClipBoardURLNew, sClipBoardURLNew.Substring(sClipBoardURLNew.IndexOf("&")), string.Empty);
                    }

                    this.comboBox1.SelectedIndex = 2;

                    if (!this.parseActive)
                    {
                        this.comboBox1.SelectedIndex = 2;

                        this.textBox1.Text = sClipBoardURLNew;

                        if (this.InvokeRequired)
                        {
                            this.Invoke((MethodInvoker)this.EnqueueJob);
                        }
                        else
                        {
                            this.EnqueueJob();
                        }
                    }
                    else
                    {
                        this.ExtractUrls.Add(sClipBoardURLNew);
                    }

                    Clipboard.Clear();
                }
            }
            catch (Exception)
            {
                //Clipboard.Clear();
            }
        }

        /// <summary>
        /// Raises the <see cref="E:Load"/> event.
        /// </summary>
        /// <param name="args">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnLoad(EventArgs args)
        {
            base.OnLoad(args);

            Application.Idle += this.OnLoaded;
        }

        /// <summary>
        /// Called when [loaded].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="args">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnLoaded(object sender, EventArgs args)
        {
            Application.Idle -= this.OnLoaded;

            this.tmrPageUpdate.Enabled = true;

#if (!PGRIPPERX)
            if (Environment.OSVersion.Platform == PlatformID.Win32NT && Environment.OSVersion.Version.Major >= 6 &&
                Environment.OSVersion.Version.Minor >= 1)
            {
                this.windowsTaskbar = TaskbarManager.Instance;
            }
#endif

            this.IdleRipper();

            try
            {
                // Reading Saved Jobs
                XmlSerializer serializer = new XmlSerializer(typeof(List<JobInfo>));
                TextReader tr = new StreamReader(Path.Combine(Application.StartupPath, "jobs.xml"));
                jobsList = (List<JobInfo>)serializer.Deserialize(tr);
                tr.Close();

                if (this.userSettings.CurrentlyPauseThreads)
                {
                    pauseCurrentThreads.Text = "(Re)Start Download(s)";
                    pauseCurrentThreads.Image = Languages.english.play;
                }

                File.Delete(Path.Combine(Application.StartupPath, "jobs.xml"));
            }
            catch (Exception)
            {
                jobsList = new List<JobInfo>();
            }

            if (jobsList.Count != 0)
            {
                StatusLabelInfo.Text = this._ResourceManager.GetString("gbParse2");
                StatusLabelInfo.ForeColor = Color.Green;

                for (int i = 0; i < jobsList.Count; i++)
                {
                    this.StatusLabelInfo.Text = string.Format("{0}{1}/{2}", this._ResourceManager.GetString("gbParse2"), i, this.jobsList.Count);
                    StatusLabelInfo.ForeColor = Color.Green;

                    ListViewItem ijobJob = new ListViewItem { Text = jobsList[i].Title };

                    ijobJob.SubItems.Add(jobsList[i].PostTitle);
                    ijobJob.SubItems.Add(jobsList[i].ImageCount.ToString());
                    listViewJobList.Items.AddRange(new[] { ijobJob });
                }

                this.working = true;

                StatusLabelInfo.Text = string.Empty;
            }

            // Delete Backup Files From AutoUpdate
            if (File.Exists("PG-Ripper.bak"))
            {
                File.Delete("PG-Ripper.bak");
            }

            // Extract Urls from Text file for Ripping
            if (File.Exists("ExtractUrls.txt"))
            {
                this.GetTxtUrls("ExtractUrls.txt");
            }
        }

        /// <summary>
        /// Extract The Cached Urls and Rip them.
        /// </summary>
        private void GetExtractUrls()
        {
            if (this.ExtractUrls.Count <= 0 && this.parseActive || this.parseActive)
            {
                return;
            }

            this.comboBox1.SelectedIndex = 2;

            this.textBox1.Text = this.ExtractUrls[0];

            if (this.InvokeRequired)
            {
                this.Invoke((MethodInvoker)this.EnqueueJob);
            }
            else
            {
                this.EnqueueJob();
            }

            this.ExtractUrls.RemoveAt(0);

            if (this.ExtractUrls.Count > 0)
            {
                this.GetExtractUrls();
            }
        }

        /// <summary>
        /// Extract Urls from Text file for Ripping
        /// </summary>
        /// <param name="sTextFolder">
        /// Path to the Text File
        /// </param>
        private void GetTxtUrls(string sTextFolder)
        {
            if (!File.Exists(sTextFolder))
            {
                return;
            }

            try
            {
                FileStream file = new FileStream(sTextFolder, FileMode.Open, FileAccess.Read);
                StreamReader sr = new StreamReader(file);
                string srRead = sr.ReadToEnd();
                sr.Close();
                file.Close();

                File.Delete(sTextFolder);

                string[] sRipUrls = srRead.Split(new[] { '\n' });

                foreach (string sRipUrl in sRipUrls)
                {
                    if (!sRipUrl.StartsWith(this.userSettings.CurrentForumUrl))
                    {
                        continue;
                    }

                    string sClipBoardURLNew = sRipUrl;

                    if (sClipBoardURLNew.Contains("\r"))
                    {
                        sClipBoardURLNew = sClipBoardURLNew.Replace("\r", string.Empty);
                    }

                    if (sRipUrl.Contains(@"&postcount=") || sRipUrl.Contains(@"&page="))
                    {
                        sClipBoardURLNew = Regex.Replace(
                            sClipBoardURLNew, sClipBoardURLNew.Substring(sClipBoardURLNew.IndexOf("&")), string.Empty);
                    }

                    this.ExtractUrls.Add(sClipBoardURLNew);
                }
            }
            finally
            {
                this.GetExtractUrls();
            }
        }

        /// <summary>
        /// Catches All Unhandled Exceptions and creates a Crash Log
        /// </summary>
        /// <param name="sender">
        /// The sender object.
        /// </param>
        /// <param name="e">
        /// The Unhandled Exception Event Arguments.
        /// </param>
        private void UnhandledExceptionFunction(object sender, UnhandledExceptionEventArgs e)
        {
            Exception ex = (Exception)e.ExceptionObject;

            Utility.SaveOnCrash(ex.Message, ex.StackTrace, currentJob);

            if (jobsList.Count > 0)
            {
                XmlSerializer serializer = new XmlSerializer(typeof(List<JobInfo>));
                TextWriter tr = new StreamWriter(Path.Combine(Application.StartupPath, "jobs.xml"));
                serializer.Serialize(tr, jobsList);
                tr.Close();

                // If Pause
                this.userSettings.CurrentlyPauseThreads = true;
                Utility.SaveSettings(this.userSettings);
            }

            if (this.userSettings.SavePids)
            {
                this.SaveHistory();
            }
        }

        /// <summary>
        /// Catches All Unhandled Thread Exceptions and creates a Crash Log
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Threading.ThreadExceptionEventArgs"/> instance containing the event data.</param>
        private void ThreadExceptionFunction(object sender, ThreadExceptionEventArgs e)
        {
            Exception ex = e.Exception;

            Utility.SaveOnCrash(ex.Message, ex.StackTrace, currentJob);

            if (jobsList.Count > 0)
            {
                XmlSerializer serializer = new XmlSerializer(typeof(List<JobInfo>));
                TextWriter tr = new StreamWriter(Path.Combine(Application.StartupPath, "jobs.xml"));
                serializer.Serialize(tr, jobsList);
                tr.Close();

                // If Pause
                if (this.userSettings.CurrentlyPauseThreads)
                {
                    Utility.SaveSettings(this.userSettings);
                }
            }

            if (this.userSettings.SavePids)
            {
                this.SaveHistory();
            }
        }

        /// <summary>
        /// Opens last downloaded Image
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void PictureBox1DoubleClick(object sender, EventArgs e)
        {
            if (this.pictureBox1.Visible && this.pictureBox1.Image != null)
            {
                System.Diagnostics.Process.Start(this.sLastPic);
            }
        }

        /// <summary>
        /// Gets the posts worker do work.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.DoWorkEventArgs"/> instance containing the event data.</param>
        private void GetPostsWorkerDoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            this.ThrdGetPosts(Convert.ToString(e.Argument));
        }

        /// <summary>
        /// Gets the posts worker completed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs"/> instance containing the event data.</param>
        private void GetPostsWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            this.UnlockControls();
        }

        /// <summary>
        /// Gets the idxs worker do work.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.DoWorkEventArgs"/> instance containing the event data.</param>
        private void GetIdxsWorkerDoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            this.ThrdGetIndexes(Convert.ToString(e.Argument));
        }

        /// <summary>
        /// Gets the idxs worker completed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs"/> instance containing the event data.</param>
        private void GetIdxsWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            this.UnlockControls();

            Application.DoEvents();

            if (this.indexedTopicsList == null)
            {
                return;
            }

            for (int po = 0; po < this.indexedTopicsList.Count; po++)
            {
                StatusLabelInfo.ForeColor = Color.Green;
                this.StatusLabelInfo.Text = string.Format("{0}{1}/{2}", "Analyse Index(es)", po, this.indexedTopicsList.Count);

                this.textBox1.Text = this.indexedTopicsList[po].ImageUrl;

                if (InvokeRequired)
                {
                    Invoke((MethodInvoker)this.EnqueueJob);
                }
                else
                {
                    this.EnqueueJob();
                }
                //////////////////////////////////////////////////////////////////////////
            }

            this.indexedTopicsList = null;
        }

        /// <summary>
        /// Genarates the Storage Folder for the Current Job
        /// </summary>
        /// <param name="curJob">Curren Job</param>
        /// <returns>The Storage Folder</returns>
        private string GenerateStorePath(JobInfo curJob)
        {
            string sStorePath = this.userSettings.DownloadFolder;

            if (this.userSettings.SubDirs)
            {
                try
                {
                    if (curJob.ForumTitle != null)
                    {
                        if (this.userSettings.DownInSepFolder)
                        {
                            sStorePath = Path.Combine(
                                this.userSettings.DownloadFolder,
                                Utility.RemoveIllegalCharecters(curJob.ForumTitle) + Path.DirectorySeparatorChar +
                                Utility.RemoveIllegalCharecters(curJob.Title) + Path.DirectorySeparatorChar +
                                Utility.RemoveIllegalCharecters(curJob.PostTitle));
                        }
                        else
                        {
                            sStorePath = Path.Combine(
                                this.userSettings.DownloadFolder,
                                Utility.RemoveIllegalCharecters(curJob.ForumTitle) + Path.DirectorySeparatorChar +
                                Utility.RemoveIllegalCharecters(curJob.Title));
                        }
                    }
                    else
                    {
                        if (this.userSettings.DownInSepFolder)
                        {
                            sStorePath = Path.Combine(
                                this.userSettings.DownloadFolder,
                                Utility.RemoveIllegalCharecters(curJob.Title) + Path.DirectorySeparatorChar +
                                Utility.RemoveIllegalCharecters(curJob.PostTitle));
                        }
                        else
                        {
                            sStorePath = Path.Combine(
                                this.userSettings.DownloadFolder, Utility.RemoveIllegalCharecters(curJob.Title));
                        }
                    }

                    int iRenameCnt = 2;

                    string sbegining = sStorePath;

                    // Auto Rename if post titles are the same...
                    if (jobsList.Count != 0)
                    {
                        string path = sStorePath;

                        foreach (JobInfo t in
                            this.jobsList.Where(
                                t =>
                                t.PostTitle.Equals(curJob.PostTitle) ||
                                Directory.Exists(path) && t.Title.Equals(curJob.Title)))
                        {
                            while (t.StorePath.Equals(sStorePath) || Directory.Exists(sStorePath))
                            {
                                sStorePath = string.Format("{0} Set# {1}", sbegining, iRenameCnt);
                                iRenameCnt++;
                            }
                        }
                    }
                    else
                    {
                        while (Directory.Exists(sStorePath))
                        {
                            sStorePath = string.Format("{0} Set# {1}", sbegining, iRenameCnt);
                            iRenameCnt++;
                        }
                    }
                }
                catch (Exception)
                {
                    sStorePath = Path.Combine(
                        this.userSettings.DownloadFolder, Utility.RemoveIllegalCharecters(curJob.Title));
                }
            }

            return sStorePath;
        }

        /// <summary>
        /// Unlock (Enable) Elements after Parsing
        /// </summary>
        private void UnlockControls()
        {
            this.parseActive = false;

            if (InvokeRequired)
            {
                this.Invoke((MethodInvoker)this.UnlockControlsElements);
            }
            else
            {
                this.UnlockControlsElements();
            }
        }

        /// <summary>
        /// Unlock the Controls
        /// </summary>
        private void UnlockControlsElements()
        {
            textBox1.Text = string.Empty;
            textBox1.Enabled = true;
            mStartDownloadBtn.Enabled = true;

            StatusLabelInfo.Text = string.Empty;
        }

        /// <summary>
        /// Unlock (Enable) Elements while Parsing
        /// </summary>
        private void LockControls()
        {
            this.parseActive = true;

            textBox1.Enabled = false;
            mStartDownloadBtn.Enabled = false;

            this.StatusLabelInfo.Text = this._ResourceManager.GetString("gbParse");
            StatusLabelInfo.ForeColor = Color.Green;
        }

        /// <summary>
        /// Enable/Disable Show last image
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void ShowLastImageToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            this.userSettings.ShowLastDownloaded = this.showLastImageToolStripMenuItem.Checked;

            this.groupBox4.Visible = this.showLastImageToolStripMenuItem.Checked;

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Enable/Disable Clipboard Monitoring
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void UseCliboardMonitoringToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            this.userSettings.ClipBWatch = this.useCliboardMonitoringToolStripMenuItem.Checked;

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Handles the CheckedChanged event of the DoNothingToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void DoNothingToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            this.closeRipperToolStripMenuItem.Checked = !this.doNothingToolStripMenuItem.Checked;

            this.userSettings.AfterDownloads = 0;

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Handles the CheckedChanged event of the CloseRipperToolStripMenuItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void CloseRipperToolStripMenuItem_CheckedChanged(object sender, EventArgs e)
        {
            this.doNothingToolStripMenuItem.Checked = !this.closeRipperToolStripMenuItem.Checked;

            this.userSettings.AfterDownloads = 1;

            Utility.SaveSettings(this.userSettings);
        }

        /// <summary>
        /// Handles the event when the download folder is changed
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void DownloadFolder_TextChanged(object sender, EventArgs e)
        {
            /*this.userSettings.DownloadFolder = this.DownloadFolder.Text;

            Utility.SaveSettings(this.userSettings);*/
        }
    }
}